#!/usr/bin/python

# session-monitor: displays information about sessions from login1
# monitors adding and removing of sessions
# see http://www.freedesktop.org/wiki/Software/systemd/logind/

import dbus, os, sys, gobject
from dbus.mainloop.glib import DBusGMainLoop
from datetime import datetime, timedelta

login1 = 0

# callback functions for login1 signals

def session_added(ses_id, ses_path): # ses_id is something like c3, ses_path '/org/freedesktop/login1/c3'
	print 'Session', ses_id, ':', ses_path,
	session = bus.get_object('org.freedesktop.login1', ses_path)
	remote = session.Get('org.freedesktop.login1.Session', 'Remote')
	if remote:
		print '(remote)'
	else:
		print '(local)'
	username = session.Get('org.freedesktop.login1.Session', 'Name')
	uid = session.Get('org.freedesktop.login1.Session', 'User')[0]
	active = session.Get('org.freedesktop.login1.Session', 'Active')
	print 'User:', username, 'UID', uid
	if active: print '(active)'
	else: print '(inactive)'
	print 'Service:', session.Get('org.freedesktop.login1.Session', 'Service')
	print 'Type:', session.Get('org.freedesktop.login1.Session', 'Type')
	print 'VT:', session.Get('org.freedesktop.login1.Session', 'VTNr')
	print 'TTY:', session.Get('org.freedesktop.login1.Session', 'TTY')
	print 'X11 Display:', session.Get('org.freedesktop.login1.Session', 'Display')
	if remote:
		print 'remote host name:', session.Get('org.freedesktop.login1.Session', 'RemoteHost')
		print 'remote user:', session.Get('org.freedesktop.login1.Session', 'RemoteUser')
	print


def session_removed(ses_id, ses_path):
	print 'Session', ses_id, 'removed :', ses_path
	print



# main

# root privilege needed for org.freedesktop.login1.ListSessions()
if os.geteuid() != 0:
	print 'root privileges needed, sorry!'

else:
# set up a DBus event loop
	dbus.mainloop.glib.DBusGMainLoop(set_as_default=True)
# connect to the system bus
	bus = dbus.SystemBus()

# create a proxy object to call methods of the login1 Manager interface
	login1 = bus.get_object('org.freedesktop.login1', '/org/freedesktop/login1')
# callback functions for signals SessionNew and SessionRemoved
	login1.connect_to_signal('SessionNew', session_added)
	login1.connect_to_signal('SessionRemoved', session_removed)

# information about existing sessions
	for session in login1.ListSessions():
		session_added(session[0], session[4])

	loop = gobject.MainLoop()
	loop.run()

